/************************************************************************/
/*  Routines to get block information from active file.			*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: get_block_info.c,v 1.4 2000/10/26 13:39:50 doug Exp $ ";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>

#include "qlib2.h"

#include "params.h"
#include "st_info.h"
#include "procs.h"
#include "externals.h"

/************************************************************************/
/*  get_block_info:							*/
/*	Get information about the next block in stream.			*/
/************************************************************************/
void get_block_info
   (ST_INFO	*st_p)		/* ptr to current stream header.	*/
{
    int	    *p;
    BS *bs;
    BLOCKETTE_HDR *bh;

    switch (st_p->hdr_type) {
    case DRM_HDR_TYPE:
	st_p->cur_hdr = decode_block_drm ((char *)st_p->iob->buf, 
					  &st_p->blksize, 
					  st_p->data_type);
	strcpy (st_p->cur_hdr->station_id, st_p->station);
	strcpy (st_p->cur_hdr->channel_id, st_p->channel);
	strcpy (st_p->cur_hdr->location_id, "");
	strcpy (st_p->cur_hdr->network_id, "");
	st_p->cur_hdr->data_type = st_p->data_type;
	break;
    case QDA_HDR_TYPE:
	st_p->cur_hdr = decode_hdr_qda ((QDA_HDR *)st_p->iob->buf, 
					st_p->iob->free);
	break;
    case SDR_HDR_TYPE:
	st_p->cur_hdr = decode_hdr_sdr ((SDR_HDR *)st_p->iob->buf,
					st_p->iob->free);
	break;
    default:
	fprintf (stderr, "unknown header type for stream %s\n", st_p->filename);
	exit(1);
	break;
    }
    if (st_p->cur_hdr == NULL) {
	fprintf (stderr, "unable to get header from stream %s\n", st_p->filename);
	exit(1);
    }
    st_p->blksize = st_p->cur_hdr->blksize;
}

/************************************************************************/
/*  NOTE:   DRM files are processed ONLY if the are in SEED_BIG_ENDIAN.	*/
/************************************************************************/

/************************************************************************/
/*  decode_block_drm:							*/
/*	Decode DRM data stored with each DRM data block,		*/
/*	and return ptr to dynamically allocated DATA_HDR structure.	*/
/*	Fill in structure with the information in a easy-to-use format.	*/
/*	WARNING:  The station_id, location_id, and channel_id are	*/
/*	NOT AVAILABLE in the block header, and are therefore are not	*/
/*	filled in at this point.  They MUST be filled in by the caller.	*/
/************************************************************************/
DATA_HDR *decode_block_drm
   (char	*buf,		/* buffer containing drm record.	*/
    int		*pblksize,	/* ptr to blksize (returned).		*/
    int		type)		/* type of drm file.			*/
{
    DATA_HDR		*ohdr;
    STORE_LOG		*sl;
    STORE_EVENT		*se;
    DA_TIME		da_time;

    if ((ohdr = new_data_hdr()) == NULL) return (NULL);
    switch (type) {
	case STYPE_LOG:
	    sl = (STORE_LOG *)buf;
	    memcpy ((void *)da_time.time_sample, (void *)sl->time_trans, 6);
	    da_time.millisec = 0;
	    ohdr->begtime = decode_time_drm(da_time, SEED_BIG_ENDIAN);
	    ohdr->endtime = ohdr->begtime;
	    break;
	case STYPE_EVENT:
	    /*	Use event time instead of log time as block time.   */
	    se = (STORE_EVENT *)buf;
	    ohdr->begtime = decode_time_drm(se->da_time, SEED_BIG_ENDIAN);
	    ohdr->endtime = ohdr->begtime;
	    break;
    }
    return (ohdr);
}
