/************************************************************************/
/*  Command line parsing routines.					*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: parse_cmdline.c,v 1.4 2004/01/25 06:35:23 doug Exp $ ";
#endif

#include <stdio.h>
#include <stdlib.h>

#include "qlib2.h"

#include "params.h"
#include "st_info.h"
#include "procs.h"
#include "externals.h"

#define	argc	(*pargc)
#define	argv	(*pargv)

/************************************************************************/
/*  parse_cmdline:							*/
/*	Parse command line.						*/
/************************************************************************/
int parse_cmdline 
   (int		*pargc,		/* ptr to argc.				*/
    char	***pargv)	/* ptr to argv.				*/
{
    char    *span;
    int	    span_flag = 0;
    char    *p;
    INT_TIME	*pt;

    /* Variables needed for getopt. */
    extern char	*optarg;
    extern int	optind, opterr;
    int		c;

    int i;
    int	nfiles;

    cmdname = tail(*argv);

    /*	Parse command line options.					*/
    while ( (c = getopt(argc,argv,"hncd:o:i:f:t:s:D:")) != -1)
	switch (c) {
	case '?':
	case 'h':   print_syntax (cmdname, syntax, info); exit(0); break;
	case 'n':   ++no_output; break;
	case 'c':   ++make_index; break;
	case 'd':   debug_option = atoi(optarg); break;
	case 'o':   outfile = optarg; break;
	case 'i':
	    /* Ignore some type of information. */
	    if (strcmp(optarg, "time") == 0) ++ignore_time;
	    else {
		fprintf (info, "invalid ignore option: %s\n", optarg);
		exit(1);
	    }
	    break;
	case 'f':
	    if ((pt = parse_date (optarg)) == NULL) {
		fprintf (info, "invalid date: %s\n", optarg);
		exit(1);
	    }
	    start_time = *pt;
	    ++start_flag;
	    break;
	case 't':
	    if ((pt = parse_date (optarg)) == NULL) {
		fprintf (info, "invalid date: %s\n", optarg);
		exit(1);
	    }
	    end_time = *pt;
	    ++end_flag;
	    break;
	case 's':
	    span = optarg; ++span_flag; break;
	case 'D':
	    /* Date display format.   */
	    if (strcmp (optarg, "j") == 0) date_fmt = JULIAN_FMT;
	    else if (strcmp (optarg, "j,") == 0) date_fmt = JULIAN_FMT_1;
	    else if (strcmp (optarg, "m") == 0) date_fmt = MONTH_FMT;
	    else if (strcmp (optarg, "m,") == 0) date_fmt = MONTH_FMT_1;
	    else {
		fprintf (info, "invalid format for date display: %s\n", optarg);
		exit(1);
	    }
	    break;
	}

    if (make_index) ++no_output;
    if (end_flag && span_flag) {
	fprintf (stderr, "span and endtime mutually exclusive\n");
	exit(1);
    }
    if (span_flag && !start_flag) {
	fprintf (stderr, "span not valid without start time\n");
	exit(1);
    }
    if (span_flag && ! valid_span(span)) {
	    fprintf (stderr, "invalid span specification: %s\n", span);
	    exit(1);
	}
    if (span_flag) {
	end_time = end_of_span (start_time, span);
	++end_flag;
    }

    if (no_output) {
	outfile = "/dev/null";
	no_output = 0;
    }
    else {
	if (outfile == NULL) info = stderr;
    }
    open_stream (&st_head_out, outfile, "w");

    /*	Skip over all options and their arguments.			*/
    argv = &(argv[optind]);
    argc -= optind;

    /* The remaining arguments are the input files.  If there are none,	*/
    /* then assume 1 file from stdin.					*/
    /* Open the files and allocate the appropriate data structures.	*/

    nfiles = argc;
    do {
	open_stream (&st_head_in, (nfiles > 0) ? *argv++ : NULL, "r");
    } while (--argc > 0);

    return(0);
}
